BaseBounceStunt = class(BaseStunt, function(self)
	self.IdleTime = 0
	self.IdleTimeMax = 2.0
	self.LastLeanState = nil
	self.Bounces = 0
	
	-- Idle state
	self.IdleState = WheelsState()
	self.IdleState.Wheels[0] = true
	self.IdleState.Wheels[1] = true
	self.IdleState.Wheels[2] = true
	self.IdleState.Wheels[3] = true
end)

function BaseBounceStunt:IsActivationConditionMet(IsActivationPending)
	-- If we are already active, don't validate (we will turn this stunt off ourselves
	-- later on in self:Process, because the deactivate condition differs from the
	-- activate condition.
	if(self.Active) then
		return true
	end
	
	local History = self:GetWheelsHistory()
	local CurrentState = History:GetRight()
	
	-- Keep track of how many lean states were found. We use this variable instead
	-- of #LeanStateIndices to save time (using # causes a linear traversal of the
	-- whole table)
	local LeanStatesFound = 0 
	local LeanStates = {}
	
	-- Find any lean states
	for i = 0, History.Count -1 do
		local State = History[History.First + i]
		if(State ~= nil and State.Flagged == false and (State == self.LeanFirstState or State == self.LeanSecondState)) then
			table.insert(LeanStates, State)
			LeanStatesFound = LeanStatesFound + 1
		end
	end
	--print("^2Leans found: " ..LeanStatesFound, "Count: " ..History.Count, "First: " ..History.First)
	
	-- If there aren't at least two lean states, there can never have been a bounce
	if(LeanStatesFound < 2) then
		return false
	end
	
	-- Verify the most recent lean state is the inverse of one of the other lean states,
	-- because we want to make sure the player is bouncing from left to right, or right
	-- to left
	local FirstLeanState = LeanStates[1]
	local Activate = false
	--self:DebugListStates()
	for i = 2, LeanStatesFound do
		local State = LeanStates[i]
		if(FirstLeanState ~= State) then
			-- A bounce has been found. Set the state flags to true to indicate all lean
			-- states up to this moment are non-readable from now on and should not be
			-- processed at a later stage. We will not return true straight away, because
			-- we need to make sure all lean states are flagged first.
			Activate = true
			State.Flagged = true
			FirstLeanState.Flagged = true -- Don't forget to flag the first state too!
		end
	end
	
	-- Either a bounce has been found and all flagging is done, or no bounce was found.
	return Activate
end

function BaseBounceStunt:Process(TimeElapsed)
	--print("^3Processing BaseBounceStunt")
	local History = self.StuntManager.WheelsHistory
	local CurrentState = History:GetRight()
	
	-- If the current state is a lean state, flag it to indicate we've seen and processed it
	if(CurrentState == self.LeanFirstState or CurrentState == self.LeanSecondState) then
		CurrentState.Flagged = true
		
		if(self.LastLeanState == nil) then
			self.LastLeanState = CurrentState
		end
		
		-- Verify if the current lean state is the inverse of the last lean state. This would
		-- mean a bounce has occurered
		if(CurrentState ~= self.LastLeanState) then
			self.Bounces = self.Bounces + 1
			self.LastLeanState = CurrentState
			self:OnBounced()
		end
	end
	
	-- Check if we should disable ourself. If the current state is the idle state,
	-- increment the idle timer.
	if(CurrentState == self.IdleState) then
		-- Increment the idle time
		self.IdleTime = self.IdleTime + TimeElapsed
		if(self.IdleTime >= self.IdleTimeMax) then
			self.IdleTime = 0
			self:Deactivate()
		end
	end
end

function BaseBounceStunt:Activate()
	self.Super.Super.Activate(self)
	
	-- We always start off with at least two bounces, because it takes two bounces to activate
	-- this stunt
	self.Bounces = 2
end

function BaseBounceStunt:Deactivate()
	self.Super.Super.Deactivate(self)
	
	self.LastLeanState = nil
	self.IdleTime = 0
	self.Bounces = 0
end

function BaseBounceStunt:OnBounced()
	--print("Total bounces: " ..self.Bounces)
end

function BaseBounceStunt:DebugListStates()
	local History = self.StuntManager.WheelsHistory
	for i = 0, History.Count -1 do
		local State = History[History.First + i]
		
		if(State == self.LeanFirstState or State == self.LeanSecondState) then
			print("#" ..i ..": " ..(State.Flagged and "true" or "false"))
		end
	end
end