BaseRotationStunt = class(BaseStunt, function(self)
	-- Misc
	self.ActivationAngle = 0
	self.DeactivationAngle = 0
	self.MaxRotationInterval = 1.5
	
	-- Rotation axis and direction
	self.RotationAxis = VectorY
	self.RotationSign = 1
	
	-- Tracking
	self.CurrentRotationInterval = 0
	self.Rotations = 0
	self.LastAngle = 0
end)

function BaseRotationStunt:IsActivationConditionMet(IsActivationPending)
	if(self.Active) then
		return true
	end
	
	local Activate = false
	local AngleDeg = self:GetAngle()
	
	-- Determine whether we need to activate the stunt or not
	--
	-- NOTE: We check whether the last angle is higher than the activation angle -10, because
	-- of problems between pitching and rolling. For instance, a 180 degree pitch is equal to
	-- a 180 degree roll. This would cause a backflip to be registered when the player rolls
	-- sideways past 180 degrees. This however also means that when the player is backflipping
	-- incredibly fast, the backflip might not be registered because the last angle will be
	-- more than 10 degrees away. However, the speed at which the player would have to flip 
	-- is unrealistically high.
	Activate = (AngleDeg > self.ActivationAngle 
		and self.LastAngle <= self.ActivationAngle 
		and self.LastAngle >= (self.ActivationAngle-10)
		and self.LastAngle >= 0)
	
	self.LastAngle = AngleDeg
	return Activate
end

function BaseRotationStunt:Process(TimeElapsed)
	local AngleDeg = self:GetAngle()
	
	-- Check for rotations. A math.sign is used to make sure the self.LastAngle isn't above
	-- the horizontal line where 179 degrees becomes -179 degrees, whilst the current
	-- angle is underneath this line. The reason why we want to make sure the last angle
	-- has the same sign as the current angle after crossing the horizontal line, is
	-- because when the truck lies on its back, it can jitter around the line, causing
	-- many rotations to be registered.
	--
	-- Example reading that would cause errors when not checking signs:
	-- Current angle: 179.7502
	-- Last angle: -179.6173
	-- 
	-- A situation like this can occur when the user goes into the opposite direction
	-- of the flip, just around the horizontal line (for instance when lying on his
	-- back and slightly moving about).
	if(AngleDeg >= self.DeactivationAngle 
		and self.LastAngle < self.DeactivationAngle
		and math.sign(self.LastAngle) == math.sign(AngleDeg)
		) then
		-- Player just finished a rotation
		self.Rotations = self.Rotations + 1
	end
	
	if(AngleDeg >= self.DeactivationAngle and AngleDeg < self.ActivationAngle) then
		-- Player is in between rotations
		self:ProcessInterval(TimeElapsed)
	else
		-- Player is currently rotating
		self.CurrentRotationInterval = 0
	end

	self.LastAngle = AngleDeg
	
end

function BaseRotationStunt:ProcessInterval(TimeElapsed)
	self.CurrentRotationInterval = self.CurrentRotationInterval + TimeElapsed
	
	if(self.CurrentRotationInterval > self.MaxRotationInterval) then
		self:Deactivate()
	end
end

function BaseRotationStunt:Activate()
	self.Super.Super.Activate(self)
	
	self.Rotations = 0
	self.CurrentRotationInterval = 0
end

function BaseRotationStunt:Deactivate()
	self.Super.Super.Deactivate(self)
end

function BaseRotationStunt:GetAngle()
	local Player = ActiveMission.HumanPlayers[1]

	local AngleDeg = 0
	if(self.RotationAxis == VectorX) then
		AngleDeg = Player:GetCarPitch()
	elseif(self.RotationAxis == VectorY) then
		AngleDeg = Player:GetCarYaw()
	elseif(self.RotationAxis == VectorZ) then
		AngleDeg = Player:GetCarRoll()
	end

	-- Apply sign
	AngleDeg = self.RotationSign * AngleDeg
	
	--print(AngleDeg)
	return AngleDeg
end