BaseStunt = class(function(self, StuntManager)
	-- Misc
	self.StuntManager = StuntManager
	self.Active = false
	self.Results = {}
	self.Distance = 0
	self.ObjectsDestroyed = 0
	
	self.ActivationStates = {}
	self.DeactivationStates = {}
	self.UseDeactivationStates = false
	
	self.Player = StuntManager.Mission.HumanPlayers[1]
	self.Car = self.Player.Car
	
	self.ID = 0
	
	-- Stunt score info
	self.Score = 0
	
	-- Air tracking
	self.InAir = false
	self.LastInAir = false
	self.MinHeight = 9e+10
	self.MaxHeight = -9e+10
	self.AirDuration = 0
	
	-- Misc delays & timing
	self.ActivationDelay = 0
	self.DeactivationDelay = 0
	self.CurrentActivationDelay = 0
	self.CurrentDeactivationDelay = 0
	self.Duration = 0
	
	-- WheelsState for save checking
	self.SavedState = WheelsState()
	self.SavedState.Wheels[0] = true
	self.SavedState.Wheels[1] = true
	self.SavedState.Wheels[2] = true
	self.SavedState.Wheels[3] = true
	
	-- WheelsState for air time checking
	self.AirState = WheelsState()
	self.AirState.Wheels[0] = false
	self.AirState.Wheels[1] = false
	self.AirState.Wheels[2] = false
	self.AirState.Wheels[3] = false
end)

function BaseStunt:AddActivationState(WheelsState)
	table.insert(self.ActivationStates, WheelsState)
end

function BaseStunt:AddDeactivationState(WheelsState)
	self.UseDeactivationStates = true
	table.insert(self.DeactivationStates, WheelsState)
end

function BaseStunt:OnCollision(DestroyableObject)
	self.ObjectsDestroyed = self.ObjectsDestroyed + 1
end

-- This is the default implementation for validating whether a stunt should be activated or not.
-- Override this method if you want to use a custom activation implementation instead.
function BaseStunt:IsActivationConditionMet(IsActivationPending)
	local CurrentState = self:GetCurrentWheelsState()
	
	if(self.UseDeactivationStates) then
		if(not self.Active) then
			-- Stunt is not yet active, iterate through the activation states
			for i = 1, #self.ActivationStates do
				local State = self.ActivationStates[i]
				if(CurrentState == State) then
					return true
				end
			end
			
			-- No suitable activation state found, return false
			return false
		else
			-- Stunt is active, iterate through the deactivation states
			for i = 1, #self.DeactivationStates do
				local State = self.DeactivationStates[i]
				if(CurrentState == State) then
					return false
				end
			end
			
			-- No suitable deactivation state found, return true to remain active
			return true
		end
	else
		-- Deactivation states aren't used, iterate through the activation states
		for i = 1, #self.ActivationStates do
			local State = self.ActivationStates[i]
			if(CurrentState == State) then
				return true
			end
		end
		
		-- No suitable activation state found, return false
		return false
	end
end

function BaseStunt:Update(ElapsedTime)
	self.IsActivationPending = self.CurrentActivationDelay > 0
	
	-- Misc updates
	self:UpdateAir(ElapsedTime)
	self:UpdateDistance(ElapsedTime)
	
	if(self:IsActivationConditionMet(self.IsActivationPending)) then
		-- The activation condition has been met
		self.CurrentDeactivationDelay = 0
		
		-- Air duration
		if(self.InAir) then
			self.AirDuration = self.AirDuration + ElapsedTime
		end
		
		-- Activation
		if(not self.Active) then
			self.CurrentActivationDelay = self.CurrentActivationDelay + ElapsedTime
			if(self.CurrentActivationDelay >= self.ActivationDelay) then
				self:Activate()
			end
		end
		
		-- Processing
		if(self.Active) then
			self:Process(ElapsedTime)
			self.Duration = self.Duration + ElapsedTime
		end
	else
		-- The activation condition was not met
		self.CurrentActivationDelay = 0
		
		if(self.Active) then
			self.CurrentDeactivationDelay = self.CurrentDeactivationDelay + ElapsedTime
			if(self.CurrentDeactivationDelay >= self.DeactivationDelay) then
				self:Deactivate()
			end
		else
			self.ObjectsDestroyed = 0
			self:ResetAirStats()
		end
	end
	
	-- History
	self.LastInAir = self.InAir

end

function BaseStunt:UpdateAir(ElapsedTime)
	local CurrentState = self:GetCurrentWheelsState()
	
	-- Air toggling
	self.InAir = CurrentState == self.AirState
	if(self.InAir and not self.LastInAir) then
		self:OnAirActivated()
	elseif(not self.InAir and self.LastInAir) then
		self:OnAirDeactivated()
	end
	
	-- Air processing
	local CurrentHeight = self.Car:GetPos().Y
	if(self.InAir and CurrentHeight > self.MaxHeight) then
		self.MaxHeight = CurrentHeight
	end
	if(self.InAir and CurrentHeight < self.MinHeight) then
		self.MinHeight = CurrentHeight
	end
end

function BaseStunt:UpdateDistance(ElapsedTime)
	-- Fetch requires matrices
	local CarMat = self.Car:GetMatrix()
	local CarRot = CarMat:GetRotation()
	local CarInvRot = CarRot:Inverse()
	
	-- Calculate the local forward speed
	local CarVel = self.Car:GetVelocity()
	local CarLocalVel = CarVel * CarInvRot
	local CarForward = VectorY:Cross(CarMat:GetRight())
	local CarLocalForward = CarForward * CarInvRot
	
	-- Calculate the adjusted speed using the forward sign
	local CarForwardZ = CarLocalVel * CarLocalForward
	local CarSpeed = CarForwardZ:Magnitude()
	--if(self.Type == StuntTypes.Wheelie) then print(CarSpeed .." ") end
	
	-- DISABLED: speed relative to direction of the car
	--local NewVel = CarLocalVel * CarLocalForward
	--local TestSpeed = NewVel:Magnitude()
	
	if((self.IsActivationPending and not self.Active) or self.Active) then
		self.Distance = self.Distance + CarSpeed
	elseif(not self.IsActivationPending and not self.Active) then
		self.Distance = 0
	end
end

function BaseStunt:Activate()
	if(self:CheckInputState())then
		-- Start the duration off at the ActivationDelay, because this is how long
		-- it took to activate the stunt.
		self.Duration = self.CurrentActivationDelay
		self.CurrentActivationDelay = 0
		self.CurrentDeactivationDelay = 0

		self.Active = true
		self:OnActivated()
	end
end

function BaseStunt:Deactivate()
	if(self:CheckInputState())then
		-- Round the duration to 2 decimals
		self.Duration = Round(self.Duration, 2)
		
		self.Active = false

		self:OnDeactivated()

		-- Reset variables (must be done after OnDeactivated is called in case
		-- a custom OnDeactivated implementation relies on these variables).
		self.CurrentActivationDelay = 0
		self:ResetAirStats()
	end
end

function BaseStunt:CheckInputState()
	local InputManager = self.Player.InputManager
	
	if(InputManager ~= nil)then
		return InputManager.Enabled
	end
	
	return false
end

function BaseStunt:ResetAirStats()
	self.AirDuration = 0
	self.MinHeight = 9e+10
	self.MaxHeight = -9e+10
end

-- Override this function for custom result configurating
function BaseStunt:CreateResult()
	local Result = StuntResult(self)
	Result.Duration = self.Duration
	Result.Distance = self.Distance
	Result.ObjectsDestroyed = self.ObjectsDestroyed
	
	if(self.AirDuration > 0) then
		Result.AirDuration = self.AirDuration
		Result.AirHeight = math.abs(self.MaxHeight - self.MinHeight)
		--print("MinHeight: " ..self.MinHeight, "MaxHeight: " ..self.MaxHeight)
	end
	
	-- Determine if saved
	Result.Saved = self:GetCurrentWheelsState() == self.SavedState
	
	return Result
end

function BaseStunt:AddResult(StuntResult)
	table.insert(self.Results, StuntResult)
	self.StuntManager:PostStuntResult(StuntResult)
end

function BaseStunt:GetGrandResult()
	local Result = StuntResult(0,0)
	for i = 1, #self.Results do
		Result = Result + self.Results[i]
	end
	
	return Result
end

function BaseStunt:GetWheelsHistory()
	return self.StuntManager.WheelsHistory
end

function BaseStunt:GetCurrentWheelsState()
	local History = self:GetWheelsHistory()
	return History:GetRight()
end

function BaseStunt:GetMission()
	return self.StuntManager.Mission
end

-- Abstract methods

function BaseStunt:Process(ElapsedTime)
	--
end

function BaseStunt:OnActivated()
	--
end

function BaseStunt:OnDeactivated()
	--
end

function BaseStunt:OnAirActivated()
	--
end

function BaseStunt:OnAirDeactivated()
	--
end

function BaseStunt:CalculateScore()
	--
end