SuspensionDataContainer = class(function(self, SuspensionManager, Car)
	self.Mesh = {}
	self.Offsets = {}
	self.Rotations = {}
	self.Matrices = {}
	self.Car = Car
	self.SuspensionManager = SuspensionManager
	
	self:Configure(Car)
end)

function SuspensionDataContainer:Configure(Car)
	local COM = VectorZero
	if (Car.GetTypeObject ~= nil) then
		COM = Car:GetTypeObject():GetChassisCOM()
	end
	
	------------------------------
	------ Fetch mesh parts ------
	------------------------------
	-- Several mesh parts have MatrixIndex = 1 to force movement with the physicsbody
	-- rather than the visual car. The reason why we want some of our parts to move
	-- with the physicsbody rather than the visual car, is because the wheels use
	-- physics for positioning (by default). We want our axles to move with our wheels,
	-- thus the axles need MatrixIndex = 1 for physics positioning.
	local Mesh = Car:GetMesh()
	
	-- Misc
	self.Mesh = {}
	self.Mesh.ShockAbsorbers = {}
	self.Mesh.ShockAbsorbers[0] = Mesh:GetObject("Schokdemper_RechtsVoor")
	self.Mesh.ShockAbsorbers[1] = Mesh:GetObject("Schokdemper_LinksVoor")
	self.Mesh.ShockAbsorbers[2] = Mesh:GetObject("Schokdemper_RechtsAchter")
	self.Mesh.ShockAbsorbers[3] = Mesh:GetObject("Schokdemper_LinksAchter")
	
	for i = 0, #self.Mesh.ShockAbsorbers do
		self.Mesh.ShockAbsorbers[i].MatrixIndex = 1
	end
	
	-- Pistons
	self.Mesh.Pistons = {}
	self.Mesh.Pistons[0] = Mesh:GetObject("Zuiger_RechtsVoor")
	self.Mesh.Pistons[1] = Mesh:GetObject("Zuiger_LinksVoor")
	self.Mesh.Pistons[2] = Mesh:GetObject("Zuiger_RechtsAchter")
	self.Mesh.Pistons[3] = Mesh:GetObject("Zuiger_LinksAchter")
	
	for i = 0, #self.Mesh.Pistons do
		self.Mesh.Pistons[i].MatrixIndex = 1
	end
	
	-- Axles
	self.Mesh.Axles = {}
	self.Mesh.Axles[0] = Mesh:GetObject("VoorAs")
	self.Mesh.Axles[1] = Mesh:GetObject("AchterAs")
	for i = 0, #self.Mesh.Axles do
		self.Mesh.Axles[i].MatrixIndex = 1
	end
	
	-- Wheels
	self.Mesh.Wheels = {}
	self.Mesh.Wheels[0] = Mesh:GetObject("Wheel_1")
	self.Mesh.Wheels[1] = Mesh:GetObject("Wheel_2")
	self.Mesh.Wheels[2] = Mesh:GetObject("Wheel_3")
	self.Mesh.Wheels[3] = Mesh:GetObject("Wheel_4")
	
	-------------------------------------
	------ Save starting situation ------
	-------------------------------------
	-- All these values (translations & rotations) will be used as the base values for all calculations
	
	-- Default axle starting rotations
	self.Rotations.Axles = {}
	self.Rotations.Axles[0] = self.Mesh.Axles[0]:GetOffsetMatrix():GetRotation()
	self.Rotations.Axles[1] = self.Mesh.Axles[1]:GetOffsetMatrix():GetRotation()
	
	-- Default axle starting positions (for Wheel.T = 0)
	self.AxleStart = self.Mesh.Axles[1]:GetOffsetMatrix():GetTranslation() 
	self.Offsets.Axles = {}
	if (self.SuspensionManager.ParentMission:is_a(FreestyleFestMission)) then
		self.Offsets.Axles[0] = Vector(0, 1.5, 0)
		self.Offsets.Axles[1] = Vector(0, 1.5, 0)
	else
		self.Offsets.Axles[0] = Vector(0, 7.5, 0)
		self.Offsets.Axles[1] = Vector(0, 7.5, 0)
	end
	
	for i = 0, #self.Offsets.Axles do
		self.Offsets.Axles[i].Y = self.Offsets.Axles[i].Y - COM.Y
	end

	-- Deep copy the axle matrices
	self.Matrices.Axles = {}
	self.Matrices.Axles[0] = Matrix34() * self.Mesh.Axles[0]:GetOffsetMatrix()
	self.Matrices.Axles[1] = Matrix34() * self.Mesh.Axles[1]:GetOffsetMatrix()
	
	-- Deep copy the piston matrices
	self.Matrices.Pistons = {}
	for i = 0, 3 do
		local Piston = self.Mesh.Pistons[i]
		self.Matrices.Pistons[i] = Matrix34() * Piston:GetOffsetMatrix()
	end
	
	-- Piston offsets
	self.Offsets.Pistons = {}
	self.Offsets.Pistons[0] = Vector(0, 3.5, 0)
	self.Offsets.Pistons[1] = Vector(0, 3.5, 0)
	self.Offsets.Pistons[2] = Vector(0, 3.5, 0)
	self.Offsets.Pistons[3] = Vector(0, 3.5, 0)
	
	-- Calculate chassis middle position
	local FirstAxlePos = self.Matrices.Axles[0]:GetTranslation()
	local SecondAxlePos = self.Matrices.Axles[1]:GetTranslation()
	local AxlePosMedian =  (FirstAxlePos + SecondAxlePos) / 2
	self.Chassis = {}
	self.Chassis.Middle = AxlePosMedian + self.Offsets.Axles[0]
	self.Chassis.Offset = Vector(0, 1, 0)
end