dofile("Lua\\Suspension\\SuspensionDataContainer.lua")

SuspensionManager = class(function(self, mission)
	self.Active = false
	self.ParentMission = mission
	
	self.UsePostPhysics = true
	self.AdjustWheelPos = true
	self.UseSingleTarget = true
end)

function SuspensionManager:SetPlayer(Player)
	self.Player = Player
	self.UseSingleTarget = true
end

function SuspensionManager:ConfigurePlayers()
	if(not self.UseSingleTarget and (self.ParentMission == nil or self.ParentMission.Players == nil)) then
		print("^1SuspensionManager ERROR: Unable to configure suspension because the associated mission has no players.")
		self:Disable()
		return
	elseif(self.UseSingleTarget and (self.Player == nil or self.Player.Car == nil)) then
		print("^1SuspensionManager ERROR: Unable to configure suspension because the player is nil or has no associated car.")
		self:Disable()
		return
	end
	
	-- Configure the parts of all players / the target player
	if(self.UseSingleTarget) then
		-- Configure the target player
		self:ConfigurePlayer(self.Player)
	else
		-- Configure all players
		for i = 1, #self.ParentMission.Players do
			self:ConfigurePlayer(self.ParentMission.Players[i])
		end
	end
end

function SuspensionManager:ConfigurePlayer(Player)
	Player.SuspensionData = SuspensionDataContainer(self, Player.Car)
end

function SuspensionManager:Enable()
	self.Active = true
	
	-- Configure all players
	self:ConfigurePlayers()
	
	-- Determine which type of PhysicsThread to use
	if(self.UsePostPhysics) then
		-- PostPhysiscsThread
		PostPhysicsThread:Spawn(function()
			while(self.Active) do
				self:DoSuspension()
				PostPhysicsThread:Wait(1)
			end
		end)
	else
		-- Normal PhysicsThread
		PhysicsThread:Spawn(function()
			while(self.Active) do
				self:DoSuspension()
				PhysicsThread:Wait(1)
			end
		end)
	end
end

function SuspensionManager:Disable()
	self.Active = false
end

function SuspensionManager:GetWheelY(Car, WheelIndex)
	local WheelT = Car:GetWheelT(WheelIndex)
	
	-- Correct vertical position when the wheel is not in contact with the ground.
	-- The engine subtracts 2 from the T-value when a wheel is not in contact with
	-- the ground for a smoother bounce animation. We want the actual T-value, so
	-- when a wheel is not in contact with the ground, we add the extra 2 again.
	if(Car:IsWheelLoose(WheelIndex) and self.AdjustWheelPos) then
		WheelT = WheelT + 2
	end
	
	return WheelT
end

function SuspensionManager:DoSuspension()
	if(self.UseSingleTarget) then
		self:DoSuspensionPlayer(self.Player)
	else
		for i = 1, #ActiveMission.Players do
			local Player = ActiveMission.Players[i]
			self:DoSuspensionPlayer(Player)
		end
	end
end

function SuspensionManager:DoSuspensionPlayer(Player)
	-- Lift the pistons
	for i = 0, 3 do
		self:LiftPiston(Player, i)
	end
	
	-- Lift the front- and rear-axles
	self:LiftAxle(Player, 0)
	self:LiftAxle(Player, 1)
	
	-- Rotate the axles to the center of the chassis
	self:RotateAxle(Player, 0)
	self:RotateAxle(Player, 1)
end

function SuspensionManager:LiftPiston(Player, WheelIndex)
	local Car = Player.Car
	local SuspensionData = Player.SuspensionData
	
	local WheelPos = self:GetWheelY(Car, WheelIndex)
	local DefaultMatrix = SuspensionData.Matrices.Pistons[WheelIndex]
	local NewTrans = DefaultMatrix:GetTranslation()
	
	-- Reposition the piston relative to the corresponding wheel
	NewTrans.Y = NewTrans.Y - WheelPos
	NewTrans = NewTrans + SuspensionData.Offsets.Pistons[WheelIndex]
	
	local PistonMesh = SuspensionData.Mesh.Pistons[WheelIndex]
	PistonMesh:SetOffsetMatrix(PistonMesh:GetOffsetMatrix():SetTranslation(NewTrans))
end

function SuspensionManager:LiftAxle(Player, AxleIndex)
	local Car = Player.Car
	local SuspensionData = Player.SuspensionData
	local AxleMesh = SuspensionData.Mesh.Axles[AxleIndex]
	
	-- Copy the axle translation (there's no easy copy by value in LUA)
	local AxleMat = SuspensionData.Matrices.Axles[AxleIndex]
	local AxleTrans = AxleMat:GetTranslation()
	
	-- Determine which wheels belong to the axle
	-- Axle 0 (front-axle) requires wheels 0 & 1
	-- Axle 1 (rear-axle) requires wheels 2 & 3
	local WheelIndexOne = (AxleIndex * 2)
	local WheelIndexTwo = (AxleIndex * 2) + 1
	
	-- Push the axle down by the median of T of both wheels
	AxleTrans.y = -((self:GetWheelY(Car, WheelIndexOne) + self:GetWheelY(Car, WheelIndexTwo)) / 2)
	
	-- Correct the new position of the axle using the AxleOffset (includes the COM)
	AxleTrans = AxleTrans + SuspensionData.Offsets.Axles[AxleIndex]
	
	AxleMesh:SetOffsetMatrix(AxleMesh:GetOffsetMatrix():SetTranslation(AxleTrans))
end

function SuspensionManager:GetAxleToWheelAngle(Player, AxleIndex, WheelIndex)
	local Car = Player.Car
	local SuspensionData = Player.SuspensionData
	local AxleMesh = SuspensionData.Mesh.Axles[AxleIndex]
	
	local WheelPos = Car:GetWheelPos(WheelIndex)
	WheelPos.Y = WheelPos.Y - self:GetWheelY(Car, WheelIndex)
	
	local AxleOffset = SuspensionData.Offsets.Axles[AxleIndex]
	local AxlePos = AxleMesh:GetOffsetMatrix():GetTranslation() + AxleOffset
	
	-- Angle calculation using inverse tangent
	local DistAxleToWheel = math.abs(WheelPos.X - AxlePos.X)
	local DistWheelStartToCurrent = math.abs(WheelPos.Y - Car:GetWheelPos(WheelIndex).Y)
	local Factor = DistWheelStartToCurrent / DistAxleToWheel
	local AngleRadians = math.atan(Factor)
	
	return AngleRadians
end

function SuspensionManager:GetAxleToChassisAngle(Player, AxleIndex)
	local Car = Player.Car
	local SuspensionData = Player.SuspensionData
	
	local AxleMesh = SuspensionData.Mesh.Axles[AxleIndex]
	local AxleOffset = SuspensionData.Offsets.Axles[AxleIndex]
	
	local AxleConnectPos = SuspensionData.Chassis.Middle + SuspensionData.Chassis.Offset
	local AxleCurrentPos = AxleMesh:GetOffsetMatrix():GetTranslation() + AxleOffset
	local AxleBasePos = Vector(0, AxleConnectPos.Y, AxleCurrentPos.Z)
	
	local DistY = AxleCurrentPos.Y - AxleBasePos.Y --math.abs(AxleCurrentPos.Y - AxleBasePos.Y)
	local DistZ = math.abs(AxleBasePos.Z - AxleConnectPos.Z)
	local Factor = DistY / DistZ
	local AngleRadians = math.atan(Factor)
	
	return AngleRadians
end

function SuspensionManager:RotateAxle(Player, AxleIndex)
	local Car = Player.Car
	local SuspensionData = Player.SuspensionData
	local AxleMesh = SuspensionData.Mesh.Axles[AxleIndex]
	
	-- Determine which wheels belong to the axle
	-- Axle 0 (front-axle) requires wheels 0 & 1
	-- Axle 1 (rear-axle) requires wheels 2 & 3
	local WheelIndexOne = (AxleIndex * 2)
	local WheelIndexTwo = (AxleIndex * 2) + 1
	
	local AngleLeft = self:GetAxleToWheelAngle(Player, AxleIndex, WheelIndexOne)
	local AngleRight = self:GetAxleToWheelAngle(Player, AxleIndex, WheelIndexTwo)
	
	-- The sign determines in which direction we're going to rotate the axle.
	-- The rear axle points to the opposite side of the front axle. Therefor, we will
	-- invert the sign for the rear axle (index 1)
	local RotationSign = 1
	if(AxleIndex == 1) then
		RotationSign = -1
	end
	
	-- Rotate the axle to line it up with the tires (Z-rotation)
	local RotationMatrix = SuspensionData.Rotations.Axles[AxleIndex]
	local Rotation = RotationSign * (AngleLeft-AngleRight)
	local NewMat = Matrix34():Rotate(Vector(0,0,1), Rotation) * RotationMatrix
	
	-- Rotate the axle to line it up with bottom-middle of the chassis (X-rotation)
	local Rotation = self:GetAxleToChassisAngle(Player, AxleIndex)
	local NewMat = NewMat:Rotate(Vector(1,0,0), RotationSign * Rotation)
	
	-- Apply default translations
	local NewMat = NewMat:SetTranslation(AxleMesh:GetOffsetMatrix():GetTranslation())
	AxleMesh:SetOffsetMatrix(NewMat)
end