BaseUpgrade = class(function(self, Upgrader)
	self.Level = 1
	self.Upgrader = Upgrader
	self.Debug = false
	self.DebugMessages = {}
end)

function BaseUpgrade:SetLevel(Level)
	if (Level < 0 or Level > 6) then
		print("^1ERROR: Attempted to set an upgrade level to " .. Level .. ". (min = 0, max = 6)")
		return
	end
	
	self.Level = Level
end

function BaseUpgrade:GetMultiplicationFactor()
	return ClampMin(0, self.Level - 1)
end

function BaseUpgrade:SetSuspension(CarBuilder, ErpDecreaseFactor, CfmDecreaseFactor)
	local MultFactor = self:GetMultiplicationFactor()
	
	-- Apply ERP and CFM tunings to all wheels
	for i = 0, 3 do
		-- Fetch a wheel
		local Wheel = CarBuilder:GetWheel(i)
		
		-- Calulcate the new ERP and CFM
		local CurrentErp = Wheel:GetERP()
		local CurrentCfm = Wheel:GetCFM()
		local ErpDecrease = -(CurrentErp * (ErpDecreaseFactor * MultFactor))
		local CfmDecrease = -(CurrentCfm * (CfmDecreaseFactor * MultFactor))
		local NewErp = CurrentErp + ErpDecrease
		local NewCfm = CurrentCfm + CfmDecrease
		
		if(self.Debug) then
			self:AddDebugMessage("Upgrading ERP of wheel #" ..i .." from "
				..string.format("%0.8f", CurrentErp) .." to " ..string.format("%0.8f", NewErp) ..".")
			self:AddDebugMessage("Upgrading CFM of wheel #" ..i .." from "
				..string.format("%0.8f", CurrentCfm) .." to " ..string.format("%0.8f", NewCfm) ..".")
		end
		
		-- Make sure the CFM & ERP are not below 0
		if(NewCfm < 0) then
			NewCfm = 0
		end
		if(NewErp < 0) then
			NewErp = 0
		end
		
		-- Apply the upgrades
		Wheel:SetERP(NewErp) -- Bounciness: hoger is meer bounce (default 0.7)
		Wheel:SetCFM(NewCfm) -- Vering: minder is minder bounce (default 0.00005)
	end
end

function BaseUpgrade:SetWheelFriction(CarBuilder, FrictionRollIncreaseFactor, FrictionSideIncreaseFactor)
	-- Apply the friction tunings to the wheels
	for i = 0, 3 do
		-- Fetch a wheel
		local Wheel = CarBuilder:GetWheel(i)
		
		-- Determine the new friction and slip values
		local CurrentFrictionRoll = Wheel:GetFrictionRoll()
		local CurrentFrictionSide = Wheel:GetFrictionSide()
		local FrictionRollIncrease = CurrentFrictionRoll * (FrictionRollIncreaseFactor)
		local FrictionSideIncrease = CurrentFrictionSide * (FrictionSideIncreaseFactor)
		local NewFrictionRoll = CurrentFrictionRoll - FrictionRollIncrease	-- Erwin: Rolwrijving moet toch juist lager zijn??
		local NewFrictionSide = CurrentFrictionSide + FrictionSideIncrease
		
		-- Make sure the two front wheels have more grip
		if(i == 2 or i == 3) then
			--NewFrictionRoll = NewFrictionRoll + (FrictionRollIncrease * 2)
		end
		
		if(self.Debug) then
			self:AddDebugMessage("Upgrading friction roll of wheel #" ..i
				.." from " ..CurrentFrictionRoll .." to " ..NewFrictionRoll ..".")
			self:AddDebugMessage("Upgrading friction side of wheel #" ..i
				.." from " ..CurrentFrictionSide .." to " ..NewFrictionSide ..".")
		end
		
		-- Apply the tunings
		Wheel:SetFrictionRoll(NewFrictionRoll)
		Wheel:SetFrictionSide(NewFrictionSide)
	end
end

function BaseUpgrade:SetWheelSlip(CarBuilder, SlipRollIncreaseFactor, SlipSideDecreaseFactor)
	-- Apply the friction and slip tunings to the BACK wheels
	for i = 0, 3 do
		-- Fetch the wheel
		local Wheel = CarBuilder:GetWheel(i)
		
		-- Determine the new friction and slip values
		local CurrentSlipRoll = Wheel:GetSlipRoll()
		local CurrentSlipSide = Wheel:GetSlipSide()
		local SlipRollIncrease = CurrentSlipRoll * (SlipRollIncreaseFactor)
		local SlipSideDecrease = -(CurrentSlipSide * (SlipSideDecreaseFactor))
		local NewSlipRoll = CurrentSlipRoll + SlipRollIncrease
		local NewSlipSide = CurrentSlipSide + SlipSideDecrease
		
		if(self.Debug) then
			self:AddDebugMessage("Upgrading slip roll of wheel #" ..i .." from "
				..string.format("%0.8f", CurrentSlipRoll) .. " to " ..string.format("%0.8f", NewSlipRoll) ..".")
			self:AddDebugMessage("Upgrading slip side of wheel #" ..i .." from "
				..string.format("%0.8f", CurrentSlipSide) .. " to " ..string.format("%0.8f", NewSlipSide) ..".")
		end
		
		-- Set the new slip roll and side to the wheel
		Wheel:SetSlipRoll(NewSlipRoll)
		Wheel:SetSlipSide(NewSlipSide)
	end
end

function BaseUpgrade:AddDebugMessage(Message)
	table.insert(self.DebugMessages, "^3   " ..Message)
	--print("^5[Upgrade] " ..Message)
end

function BaseUpgrade:PrintDebugMessages()
	for i = 1, #self.DebugMessages do
		local Message = self.DebugMessages[i]
		print(Message)
	end
	
	-- Clear the messages
	self.DebugMessages = {}
end

-- Abstract methods

function BaseUpgrade:Apply(CarBuilder)
	--print("^1ERROR: Attempted to call apply on an empty Upgrade.")
end